//
//  IKAROS.h        Kernel code for the IKAROS project
//                        Version 1.0.0
//
//    Copyright (C) 2001-2007  Christian Balkenius
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//    See http://www.ikaros-project.org/ for more information.
//
//    Created July 13, 2001
//

#ifndef IKAROS
#define IKAROS


#define VERSION "1.0.0"

#include "IKAROS_System.h"
#include "Kernel/IKAROS_Timer.h"
#include "Kernel/IKAROS_Socket.h"
#include "Kernel/IKAROS_Utils.h"
#include "Kernel/IKAROS_Math.h"
#include "Kernel/IKAROS_Serial.h"
#include "Kernel/IKAROS_Threads.h"
#include "Kernel/IKAROS_XML.h"

// Messages to use with Notify

const int    msg_fatal_error	=    1;
const int    msg_exception		=    2;
const int    msg_warning		=    3;
const int    msg_print		=    4;
const int    msg_end_of_file	=    5;
const int    msg_terminate		=    6;
const int    msg_verbose		=    7;

// Print modes for Notify

const int    print_silent		=    0;
const int    print_normal		=    msg_print;
const int    print_verbose		=    msg_verbose;


const int    unknown_size        =    -1;

class Module;
class Kernel;
class WebUI;



//
// Parameter is used during the creation of a module to set internal values
//


class Parameter
{
public:
    Kernel * kernel;
    XMLElement * xml;

    Parameter(Kernel * k, XMLElement * x)
    {
        kernel = k;
        xml = x;
    }

// THESE FUNCTIONS DO NOT WORK - THEY ARE KEPT HERE TO MAKE SOME OLD MODULES COMPILE *** MUST BE CHANGED ***

    char *            GetName()
    {
        printf("The module uses a parameter function that has been removed from Ikaros. The module has to be modified before it can be used.\n");
        return NULL;
    }
    char *            GetValue()
    {
        printf("The module uses a parameter function that has been removed from Ikaros The module has to be modified before it can be used.\n");
        return NULL;
    }
    Parameter *        GetNext()
    {
        printf("The module uses a parameter function that has been removed from Ikaros The module has to be modified before it can be used.\n");
        return NULL;
    }
    Parameter *        GetNext(char * n)
    {
        printf("The module uses a parameter function that has been removed from Ikaros The module has to be modified before it can be used.\n");
        return NULL;
    }
    Parameter *        GetParameters()
    {
        printf("The module uses a parameter function that has been removed from Ikaros The module has to be modified before it can be used.\n");
        return NULL;
    }
};



//
// Class is used to hold pointers to module creators
//

typedef Module * (*ModuleCreator)(char *, Parameter *);



class Class
{
private:
    ModuleCreator    module_creator;
public:
    char *            name;
    char *            path;
    Class *           next;

    Class(char * n, ModuleCreator mc, char * p, Class * nxt = NULL);
    ~Class();

    char *                GetClassPath(char * class_name);
    friend Module *       CreateModule(Class * c, char * class_name, char * n, Parameter * p);        // Return a module of class class_name initialized with parameters in p and attributes in a
};



//
// Module_IO is used internally by Module and Kernel to represent the input and output connections of a module
//

class Module_IO
{
public:
    int                sizex;        // no of columns    *** temporarily made public for WenUI ***
    int                sizey;        // no of rows
    float        ***        matrix;    // matrix version of data; array of pointers to columns; Array of matrixes in 0.8.0 for delays
private:
    Module_IO(Module_IO * nxt, Module * m, char * n, int x=unknown_size, int y=1);        // Create output from module m with name n and size x and y
    ~Module_IO();                                                            // Deletes the data

    void                Allocate();
    void                SetSize(int x, int y=1);
    void                DelayOutputs();


    Module_IO   *   next;
    Module      *   module;
    char        *   name;
    float       **  data;        // Array in 0.8.0 for delays
    int             size;        // should equal sizex*sizey
    int             max_delay;  // maximum number of arrays/matrices

    friend class Kernel;
    friend class Connection;
    friend class Module;
};



//
// Module is the base class for all simulation modules
//

class Module
{
public:
    char *    GetName();                                // Get the name of the module assigned during creation
    char *    GetClassName();

    float *    GetInputArray(char * name, bool required=true);                  // Get a pointer to the input array of this module with a certain name
    float *    GetOutputArray(char * name, bool required=true);                 // Get a pointer to the output array of this module with a certain name

    float **    GetInputMatrix(char * name, bool required=true);                // Get a pointer to the input matrix of this module with a certain name
    float **    GetOutputMatrix(char * name, bool required=true);               // Get a pointer to the output matrix of this module with a certain name

    int        GetInputSize(char * input_name);               // Get the size of an input array (size_x * size_y if two-dimensional)
    int        GetInputSizeX(char * name);                    // Get the horizontal size of an input array
    int        GetInputSizeY(char * name);                    // Get the vertical size of an input array (1 if one-dimensional)

    int        GetOutputSize(char * name);                    // Get the size of an output array (size_x * size_y if two-dimensional)
    int        GetOutputSizeX(char * name);                   // Get the horizontal size of an output array
    int        GetOutputSizeY(char * name);                   // Get the vertical size of an output array (1 if one-dimensional)

    bool        InputConnected(char * name);                // True if input receives at least one connection
    bool        OutputConnected(char * name);               // True if output is connected to at least one module

    virtual void    SetSizes()
    {}        // Calculate and set the sizes of unknown output arrays    (OVERRIDE IN SUBCLASSES)
    virtual void    Init()
    {}            // Init memory for internal data                        (OVERRIDE IN SUBCLASSES)
    virtual void    Tick()
    {}            // Update state of the module                            (OVERRIDE IN SUBCLASSES)

    void            Notify(int msg);                    // Send message to the kernel (for example terminate or error, using constants defined above)
    void            Notify(int msg, char *format, ...);    // Send message to the kernel and print a massage to the user

protected:
    void            AddInput(char * name);
    void            AddOutput(char * name, int size_x=unknown_size, int size_y=1);    // Allocate output

//    void            SetClassName(char * name);

    void            SetOutputSize(char * name, int size_x, int size_y=1);    // Set the output size for an output of unknown size; it is an error to change the output size

    Module(char * n);            // Creates the module            DEPRECATED
    Module(Parameter * p);        // Creates the module            (MUST BE CALLED FROM SUBCLASS CREATOR)

    virtual            ~Module();

    // Get values from parameters in XML tree - implements the semantics of the ikc file format

    char *        GetValue(char * n);                            // Search through XML for parameter and return its value or NULL
    float        GetFloatValue(char * n, float d=0);            // Search through XML for parameter and return its value as a float or default value d if not found
    int            GetIntValue(char * n, int d=0);                // Search through XML for parameter and return its value as a float or default value d if not found
    bool        GetBoolValue(char * n, bool d=false);        // Search through XML for parameter and return its value as a float or default value d if not found
    int            GetIntValueFromList(char * n, char * list);    // Search through XML for parameter and then search list for the index of the value in the parameter; return 0 if not foun
    float *	GetArray(char *n, int size);		// Search through XML for parameter and return its value as an array
    float **	GetMatrix(char *n, int sizex, int sizey);	// Search through XML for parameter and return its value as a matrix

    // Functions for backward compatibility - ignore p

    char *        GetValue(Parameter * p, char * n)
    {
        return GetValue(n);
    } // DEPRECATED
    float        GetFloatValue(Parameter * p, char * n, float d=0)
    {
        return GetFloatValue(n, d);
    } // DEPRECATED
    int            GetIntValue(Parameter * p, char * n, int d=0)
    {
        return GetIntValue(n, d);
    } // DEPRECATED
    bool        GetBoolValue(Parameter * p, char * n, bool d=false)
    {
        return GetBoolValue(n, d);
    } // DEPRECATED
    int            GetIntValueFromList(Parameter * p, char * n, char * list)
    {
        return GetIntValueFromList(n, list);
    } // DEPRECATED
    Parameter * GetParameters(Parameter * p, char * n)
    {
        printf("The module uses a parameter function that has been removed from Ikaros The module has to be modified before it can be used.\n");
        return NULL;
    };

    XMLElement *    xml;

private:
    Module     *        next;                    // Next module in list
    Module     *        next_in_threadGroup;    // Next module in ThreadGroup

    char    *        class_name;
    char *            name;

    Kernel    *        kernel;

    Module_IO    *    input_list;            // List of inputs
    Module_IO    *    output_list;        // List of outputs

    int                period;            // How often should the module tick
    int                phase;            // Phase when the module ticks

    Timer *            timer;            // Statistics variables
    float            time;
    float            ticks;

    void            DelayOutputs();

    Module_IO *        GetModule_IO(Module_IO * list, const char * name);
    void            AllocateOutputs();                                // Allocate memory for outputs

    friend class Kernel;
    friend class Module_IO;
    friend class Connection;
    friend class ThreadGroup;

    friend Module *        CreateModule(Class * c, char * class_name, char * n, Parameter * p);
};



//
// Connection is used internally by Kernel to store pointers to data in Module_IO that are copied during Propagate()
//

class Connection
{                            // Connection contains info about data flow between modules
private:
    Connection        *    next;                // Next connection in list
    Module_IO        *    source_io;
    int                    source_offset;
    Module_IO        *    target_io;
    int                    target_offset;
    int                    size;                // The size of the data to be copied
    int                    delay;

    Connection(Connection * n, Module_IO * sio, int so, Module_IO * tio, int to, int s, int d = 1);
    ~Connection();

    void Propagate(long tick);

    friend class Kernel;
};



//
// ThreadGroup contains a linked list of modules that are executed in the same thread
//

class ThreadGroup
{
public:
    Kernel *        kernel;
    ThreadGroup *    next;
    Module *        modules;
    Module *        last_module;        // Last module in list

    int            period;            // How often should the thread be started
    int            phase;            // Phase when the thread should start

    Thread *        thread;

    void        AddModule(Module * m);        // Try to add module to this or next thread; return false if it is necessary to create a new thread

    ThreadGroup(Kernel * k);
    ~ThreadGroup();

    void        Start(long tick);
    void        Stop(long tick);
    void        Tick();
};



//
// Node is a collection of running nodes
//

class Node
{
public:
    char *	name;
    char *	host;
    int		port;
    Node *	next;

    Node();
    ~Node();
};



//
// Kernel is the main object and points to data structures for modules and connections
//

class Kernel
{
public:
    FILE *      logfile;
    char *      ikaros_dir;
    char *      ikc_dir;
    char *      ikc_file_name;

    Options *   options;

    Kernel(Options * opt);
    ~Kernel();

    void        AddClass(char * name, ModuleCreator mcc, char * path = NULL);    // Add a new class of modules to the kernel

    void        Notify(int msg, char *format, ...);

    bool        Terminate();                                // True if the simulation should end
    void        Run();                                      // Run the simulation until kernel receives notification; for example end-of-file from a module

    void        Init();                                     // Call Init() for all modules
    void        Tick();                                     // Call Tick() for all modules

    void        AddModule(Module * m);                      // Add a module to the simulation
    Module *    GetModule(const char * n);                  // Find a module based on its name

    void        Connect(const char * source_module_name, const char * source_output_name, const char * taget_module_name, const char * target_input_name, int d=1);    // Connect output to input

    bool        GetSource(XMLElement * group, Module * &m, Module_IO * &io, char * source_module_name, char * source_name);
    bool        GetTarget(XMLElement * group, Module * &m, Module_IO * &io, char * target_module_name, char * target_name);

    void        BuildGroup(XMLElement * xml, char * current_class = NULL);
    void        ReadXML(char * filename);
    void        ReadCompatibilityXML();

    bool        InputConnected(Module * m, char * input_name);
    bool        OutputConnected(Module * m, char * output_name);

    long        GetTick()
    {
        return tick;
    }

    void        ListInfo();
    void        ListModulesAndConnections();
    void        ListScheduling();
    void        ListThreads();
    void        ListClasses();
    void        ListProfiling();
    void        PrintTiming();                 // Print total timing information

private:
    Class        *     classes;            // List of module classes
    long                 tick;            // Updated every iteration
    long                 max_ticks;        // Max iterations, stop after these many ticks

    long                 tick_length;    // Desired length (in ms) of each tick
    Timer *            timer;            // Global timer
    float                  total_time;        // Total execution time at termination

    int                  print_mode;     // How much to print

    XMLDocument     *    xmlDoc;

    Module          *    modules;                    // List of modules
    Connection      *    connections;                // List of connections
    bool                 useThreads;

    ThreadGroup     *    threadGroups;                // List of ThreadGroups

    // Execution Control

    int         module_count;
    int         period_count;
    int         phase_count;

    bool        end_of_file_reached;                        // Flags set on notification from modules
    bool        fatal_error_occured;
    bool        terminate;

    bool        sizeChangeFlag;

    void        CheckInputs();                                // Check that memory for all connected inputs have been allocated
    void        CheckOutputs();                               // Check that all outputs are correctly set

    bool        Preceedes(Module * a, Module * b);
    void        SortModules();                                // Sort modules in precedence order
    void        CalculateDelays();                            // Calculate the maximum delay from each output

    void        InitInputs();                                 // Allocate memory for inputs in all modules
    void        InitOutputs();                                // Calculate and set output sizes that were not set at module creation; called by Init()
    void        AllocateOutputs();                            // Allocate memory for outputs
    void        InitModules();                                // Init all modules; called by Init()

    int         CalculateInputSize(Module_IO * i);             // Calculate the size of the input using connections to it
    int         CalculateInputSizeX(Module_IO * i);            // Calculate the size of the input using connections to it
    int         CalculateInputSizeY(Module_IO * i);            // Calculate the size of the input using connections to it

    void        NotifySizeChange();

    void Propagate();                                    // Copy output data to input for all modules
    void DelayOutputs();

    friend class Module;
    friend class Module_IO;
    friend class ThreadGroup;
    friend class WebUI;
};



#endif


