//
//	IKAROS_Socket.h		Socket utilities for the IKAROS project
//
//    Copyright (C) 2001-2006  Christian Balkenius
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//    See http://www.ikaros-project.org/ for more information.
//


#ifndef IKAROS_SOCKET
#define IKAROS_SOCKET

#include "IKAROS_System.h"

#ifdef USE_SOCKET

#define PORTNO		8000 	// the default port users will be connecting to



class SocketException
{
public:
    char	*		string;
    int			internal_reference;

    SocketException(char * s, int ref = 0) : string(s), internal_reference(ref)
    {};
};



//
//	Socket used to request data from a HTTP server in a simple way
//

class SocketData;

class Socket
{
public:
    Socket();
    ~Socket();

    bool				SendRequest(char * hostname, int port, char * request);
    bool				Poll(); // return true if data is waiting
    int				ReadData(char * result, int maxlen, bool fill=false);
    int				Get(char * hostname, int port, char * request, char * result, int maxlen);
    void				Close();

private:
    SocketData *		data;
};



//
//	The class socket implements a simple HTTP server
//

class ServerSocketData;

class ServerSocket
{
public:
    ServerSocket(int port=PORTNO);				// Create server socket on port
    ~ServerSocket();

    bool				GetRequest();								// Poll for request and read if available

    bool				TestArgument(int i, char * s);					// Test argument of last request
    bool				TestArgumentExtension(int i, char * s);			// Test the type of the arguement as a file name

    bool				TestPath(int i, char * s);						// Test start of path in argument
    char *			GetArgument(int i);							// Get part of request

    bool				SendHTTPHeader(char * type, long size = -1);
    bool				SendData(char * buffer, long size);
    bool				Send(char * format, ...);
    bool				Send(float *, int sizex, int sizey);
    bool				SendFile(char * path, char * filename);

    bool				Close();									// Close reply socket

    int				Port();									// Get port no
    char *			Host();									// Get host name

    int				errcode;									// Last error
private:
    ServerSocketData *	data;

    bool				Poll();									// Poll for connection; return >=0 if accepted connection (or > 0 CHECK!!!)
    int				Read(char * buffer, int maxSize);				// Read
};


#endif
#endif
