//
//	IKAROS_Utils.h		Various utilities for the IKAROS project
//
//    Copyright (C) 2001-2007  Christian Balkenius
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//    See http://www.ikaros-project.org/ for more information.
//
//	Created: July 17 2001
//

#ifndef IKAROS_UTILS
#define IKAROS_UTILS

#include <stdlib.h>

// Basic string functions

char *	create_string(char * c); // copy string c
char *  create_string_head(char * c, int n); // copy at most n characters
char *  create_string_tail(char * c, int i); // copy starting at character i
char *	create_formatted_string(char * format, ...);
void    destroy_string(char *c);

bool    equal_strings(const char * a, const char * b);

// File path operations

bool    is_path(char * p);
bool    is_absolute_path(char * p);

// Array and matrix functions

void        print_array(char * name, float * a, int size);
void        print_matrix(char * name, float ** m, int sizex, int sizey);

float *     create_array(int size);
float **    create_matrix(int sizex, int sizey);

void        destroy_array(float * a);
void        destroy_matrix(float ** a);

float *     copy_array(float * r, float * a, int size);
float **    copy_matrix(float ** r, float ** a, int sizex, int sizey);

float *     reset_array(float * a, int size);
float **	reset_matrix(float ** m, int sizex, int sizey);

float *     set_array(float * a, float v, int size);
float **    set_matrix(float ** m, float v, int sizex, int sizey);




// Delay Line

class DelayLine
{
public:
    int size;
    int pos;
    float * data;

    DelayLine(int s) : size(s), pos(0)
    {
        data = new float[s];
    }	// s > 0; s = 1 is no delay
    ~DelayLine()
    {
        delete [] data;
    }
    void	put(float v)
    {
        data[pos] = v;
        pos = (pos + 1) % size;
    }
    float	get()
    {
        return data[(pos + 1) % size];
    }
};



// Options

class Options
{
public:
    Options(int argc, char *argv[]);
    ~Options();

    void    SetOption(char c, char * arg = NULL);
    bool    GetOption(char c);
    char *  GetArgument(char c);

    char *  GetWorkingDirectory();
    char *  GetBinaryDirectory();
    
    char *  GetFilePath();
    char *  GetFileDirectory();
    char *  GetFileName();
    
    void    Print();
private:
    bool    option[256];
    char *  argument[256];
    char    working_dir[1024];
    char *  binary_dir;
    char *  file_path;
    char *  file_dir;
    char *  file_name;
};



#endif

