//
//	InputQTMovie.cc		This file is a part of the IKAROS project
//						A module for reading from files with data in column form
//
//    Copyright (C) 2001-2003  Christian Balkenius
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include "InputQTMovie.h"

#ifdef USE_QUICKTIME

using namespace qt;

#define	BailNULL(n)  if (!n) goto bail;
#define	BailError(n) if (n) goto bail;



typedef NavObjectFilterUPP	QTFrameFileFilterUPP;



//------------------------------------------------------------------------------
//	Open up the Movie file and get a Movie from it.
//------------------------------------------------------------------------------
void
InputQTMovie::OpenMovie(UInt8 * filename)
{
    FSRef	theFSRef;
    FSSpec	theFSSpec;
    short	refnum = 0;
    Boolean  isDirectory;

    OSErr err = noErr;

    err =FSPathMakeRef (filename, &theFSRef, &isDirectory);
    BailError(err);

    err =FSGetCatalogInfo (&theFSRef, kFSCatInfoNone, NULL, NULL, &theFSSpec, NULL);
    BailError(err);

    err = OpenMovieFile(&theFSSpec, &refnum, fsRdPerm);
    BailError(err);

    err = NewMovieFromFile(&movie, refnum, NULL, NULL, newMovieActive, NULL);
    BailError(err);

bail:
    if (refnum)
        CloseMovieFile(refnum);
}



void
InputQTMovie::GetNextFrame()
{
    CGrafPtr	 	savedPort;
    GDHandle     	savedDevice;
    Rect 		 	theRect;

    SetMovieGWorld(movie, movieGWorld, GetGWorldDevice(movieGWorld));

    GetGWorld(&savedPort, &savedDevice);
    SetGWorld(movieGWorld, NULL);

    // (gFrameNumber == 0 ? nextTimeEdgeOK : 0)

    GetMovieNextInterestingTime(movie, nextTimeStep, 0, NULL, movieTime, fixed1, &movieTime, NULL);

    if (movieTime != -1)
    {
        int errCode = GetMoviesError();
        if (errCode != noErr)
        {
            Notify(msg_warning, "QuickTime ErrCode = %d (t = %d)\n", errCode, movieTime);
        }

        // set the time for the frame and give time to the movie toolbox
        SetMovieTimeValue(movie, movieTime);
        GetPortBounds(movieGWorld, &theRect);
        EraseRect(&theRect);	// paint the background black WHY???

        MoviesTask(movie, 0);  // this does the actual drawing into the GWorld
    }

    else if (loop)
    {
        movieTime = 0;
        GetMovieNextInterestingTime(movie, nextTimeStep | nextTimeEdgeOK, 0, NULL, movieTime, fixed1, &movieTime, NULL);	// RESTART MOVIE

        // set the time for the frame and give time to the movie toolbox
        SetMovieTimeValue(movie, movieTime);
        GetPortBounds(movieGWorld, &theRect);
        EraseRect(&theRect);	// paint the background black WHY???

        MoviesTask(movie, 0);  // this does the actual drawing into the GWorld

    }

    else
        Notify(msg_end_of_file);

    SetGWorld(savedPort, savedDevice);
}



InputQTMovie::InputQTMovie(char * name, Parameter * p):
        Module(p)
{
    movieGWorld = NULL;
    movie = NULL;
    movieTime = 0;			// set current time value to begining of the Movie

    filename = GetValue(p, "filename");
    loop = GetBoolValue(p, "loop", false);

    if (!filename)
    {
        Notify(msg_fatal_error, "No filename for InputQTMovie.\n");
        return;
    }

    size_x  = GetIntValue(p, "size_x", -1);		// Default is to get the output size from the movie itself
    size_y  = GetIntValue(p, "size_y", -1);

    Rect rectMovie;

    OSErr result = noErr;

//	InitCursor(); deprecated 10.4
    result = EnterMovies();
    if (result != noErr)
        return;

    // get the movie

    OpenMovie((UInt8 *)filename);
    if (!movie)
        return;

    // normalize the movie rect
    GetMovieBox(movie, &rectMovie);

    OffsetRect(&rectMovie, -rectMovie.left, -rectMovie.top);

    Notify(msg_verbose, "MovieSize: %d %d\n", rectMovie.right, rectMovie.bottom);

    SetMovieBox(movie, &rectMovie);
    movieBox = rectMovie;

    if (size_x == -1 )
    {
        size_x = rectMovie.right;
        size_y = rectMovie.bottom;
    }
    else	// Set up transformation matrix
    {
        MatrixRecord	scaleMatrix;
        Rect			dest = {0, 0, size_y, size_x};
        RectMatrix(&scaleMatrix, &rectMovie, &dest);
        SetMovieMatrix(movie, &scaleMatrix);
    }

    // create the offscreen to draw into

    Rect srcRect= { 0, 0, size_y, size_x };

    Rect portRect;
    RGBColor theBlackColor = { 0, 0, 0 };
    RGBColor theWhiteColor = { 65535, 65535, 65535 };

    result = NewGWorld(&movieGWorld, k32ARGBPixelFormat, &srcRect, NULL, NULL, 0);
    BailError(result);

    LockPixels(GetGWorldPixMap(movieGWorld));

    SetGWorld(movieGWorld, NULL);
    GetPortBounds(movieGWorld, &portRect);
    RGBBackColor(&theBlackColor);
    RGBForeColor(&theWhiteColor);
    EraseRect(&portRect);
    BailNULL(movieGWorld);

//	CountThemFrames();

    AddOutput("INTENSITY", size_x, size_y);
    AddOutput("RED", size_x, size_y);
    AddOutput("GREEN", size_x, size_y);
    AddOutput("BLUE", size_x, size_y);

bail:
    return;
}



InputQTMovie::~InputQTMovie()
{
    if (movie)
        DisposeMovie(movie);
}



Module *
InputQTMovie::Create(char * name, Parameter * p)
{
    return new InputQTMovie(name, p);
}



void
InputQTMovie::Init()
{
    Notify(msg_warning, "InputQTMovie is not thread safe\n");

    intensity		= GetOutputArray("INTENSITY");
    red			= GetOutputArray("RED");
    green		= GetOutputArray("GREEN");
    blue			= GetOutputArray("BLUE");
}



void
InputQTMovie::Tick()
{
    GetNextFrame();

    // Copy to IKAROS output arrays

    PixMapHandle	portPixMap = GetPortPixMap(movieGWorld);

    BytePtr    dataPixels = (BytePtr)(**portPixMap).baseAddr;
    int		w =  (**portPixMap).bounds.right - (**portPixMap).bounds.left;
    int		h = (**portPixMap).bounds.bottom - (**portPixMap).bounds.top;
    int		rb = GetPixRowBytes(portPixMap);

    long		ix = 0;
    for (int y=0; y<h; y++)
    {
        BytePtr p = dataPixels + y*rb + 1;
        for (int x=0; x<w; x++)
        {
            red[ix]		= (*p)/255.0; p++;
            green[ix]		= (*p)/255.0; p++;
            blue[ix]		= (*p)/255.0;
            p++;
            p++;
            intensity[ix] 	= (red[ix] +green[ix] + blue[ix])/3.0;
            ix++;
        }
    }
}

#endif
