//
//	InputVideoQT.cc		This file is a part of the IKAROS project
//                                                      A module to grab image data from a QuickTime compatible camera
//
//    Copyright (C) 2002  Christian Balkenius
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// 	NOTE: QuickTime Specific Code
//
// 	This code must be compiled with the Carbon and QuickTime frameworks on OS X
// 	May possibly be compiled with QuickTime for Windows
//
// 	Assumes that we get image data in yuv2 format
// 	This is the case for Logitech QuickCam Pro 3000
//
//	See SGDataProcSample at developer.apple.com for info on how to make it more
//	compatible with other cameras/inputs using decompression to a standard format.
//

#include "InputVideoQT.h"

#ifdef USE_QUICKTIME

#include <string.h>

//
// Grabber
//

namespace qt {
#include <Carbon/Carbon.h>
#include <QuickTime/QuickTime.h>
}

using namespace qt;

#include <stdio.h>

#define IMGBUFSIZE	307200

typedef struct {
    SeqGrabComponent	seqGrab;
    SGChannel			sgchanVideo;
    GWorldPtr			pGWorld;
    Rect					boundsRect;
    int					index;
    bool					grabDone;
    unsigned char *		data;
    size_t				dataSize;
    ImageSequence		decomSeq;
} tGrabberDataRecord, *GrabberPtr;



static OSErr					Grabber_Open(GrabberPtr * g, int x, int y);
static OSErr					Grabber_Close(GrabberPtr g);
static OSErr					Grabber_Grab(GrabberPtr g);
//static OSErr					Grabber_WriteBWToFile(GrabberPtr g, char * fileName);
//static unsigned char *			Grabber_GetData(GrabberPtr g);
//static size_t					Grabber_GetDataSize(GrabberPtr g);


#define BailErr(x) { err = x; if(err != noErr) goto bail; }


pascal OSErr mySGDataProc(SGChannel c, Ptr p, long len, long *offset, long chRefCon, TimeValue time, short writeType, long refCon);
pascal OSErr mySGDataProc(SGChannel c, Ptr p, long len, long *offset, long chRefCon, TimeValue time, short writeType, long refCon)
{
#pragma unused(offset,chRefCon,time,writeType)
    CodecFlags              ignore;
    ComponentResult     err = noErr;

    GrabberPtr 	g = (GrabberPtr)refCon;

    if (g && g->decomSeq)
    {
        err = DecompressSequenceFrameS(g->decomSeq, p, len, 0, &ignore, NULL);
        if (err != noErr)
        {
            return 0;
        }
    }

    // ******  IMAGE IS NOW IN THE GWORLD ****** //

    if (g->grabDone)
        return noErr;
    /*
    	if(g->dataSize != (unsigned long)len)
    	{
    		if(g->data != NULL)
    			free(g->data);
    		g->data = (unsigned char *)malloc(len);
    		g->dataSize = len;
    	}

    	memcpy(g->data, p, len);
    */
    g->grabDone = true;

    return noErr;
}



static OSErr
Grabber_Open(GrabberPtr * g, int x, int y)
{
    OSErr				err = noErr;
    GrabberPtr			gg;
    Rect					sourceRect = { 0, 0 };			// Is this different from below???
    Rect					rect = { 0, 0, y, x };				// FixMe: Check that rect is ok later!!!
    ImageDescriptionHandle	imageDesc = (ImageDescriptionHandle)NewHandle(0);
    MatrixRecord			scaleMatrix;

    // Misc Inits

    InitCursor();
    EnterMovies();

    // Initialize Grabber Data

    *g = NULL;
    gg = (GrabberPtr)calloc(1, sizeof(tGrabberDataRecord));
    if (gg ==NULL) return -1;
    *g = gg;

    gg->seqGrab = NULL;
    gg->sgchanVideo = NULL;
    gg->index = 0;
    gg->grabDone = false;
    gg->boundsRect = rect;
    gg->pGWorld = NULL;
    gg->data = NULL;
    gg->dataSize = 0;

    err = QTNewGWorld (&(gg->pGWorld), k32ARGBPixelFormat, &(gg->boundsRect), NULL, NULL, 0); BailErr(err);

    if (!LockPixels(GetPortPixMap(gg->pGWorld)))
    {
        BailErr(-1);
    }

    // Initialize Sequence Grabber

    err = OpenADefaultComponent(SeqGrabComponentType, 0, &(gg->seqGrab)); BailErr(err);
    err = SGInitialize(gg->seqGrab); BailErr(err);
    err = SGSetDataRef(gg->seqGrab, 0, 0, seqGrabDontMakeMovie); BailErr(err);
    err = SGSetGWorld(gg->seqGrab, gg->pGWorld, GetMainDevice()); BailErr(err);		// GetMainDevice may be replaced by NULL???
    err = SGNewChannel(gg->seqGrab, VideoMediaType, &(gg->sgchanVideo)); BailErr(err);
    err = SGSetChannelBounds(gg->sgchanVideo, &(gg->boundsRect)); BailErr(err);
    err = SGSetChannelUsage(gg->sgchanVideo, seqGrabRecord); BailErr(err);
    err = SGSetDataProc(gg->seqGrab, NewSGDataUPP(mySGDataProc), (long)gg); BailErr(err);
    err = SGStartRecord(gg->seqGrab); BailErr(err);

    // Setup decompression sequence

    err = SGGetChannelSampleDescription(gg->sgchanVideo, (Handle)imageDesc); BailErr(err);

    sourceRect.right = (**imageDesc).width;
    sourceRect.bottom = (**imageDesc).height;

//	printf("sourceRect: %d %d %d %d\n", sourceRect.top, sourceRect.left, sourceRect.bottom, sourceRect.right);
//	printf("boundsRect: %d %d %d %d\n", gg->boundsRect.top, gg->boundsRect.left, gg->boundsRect.bottom, gg->boundsRect.right);

    RectMatrix(&scaleMatrix, &sourceRect, &(gg->boundsRect));

    err = DecompressSequenceBegin(&gg->decomSeq,	// pointer to field to receive unique ID for sequence
                                  imageDesc,			// handle to image description structure
                                  (gg->pGWorld),			// port for the DESTINATION image
                                  NULL,				// graphics device handle, if port is set, set to NULL
                                  NULL,				// source rectangle defining the portion of the image to decompress
                                  &scaleMatrix,			// transformation matrix
                                  srcCopy,				// transfer mode specifier
                                  NULL,				// clipping region in dest. coordinate system to use as a mask
                                  0,					// flags
                                  codecNormalQuality,		// accuracy in decompression
                                  bestSpeedCodec);		// compressor identifier or special identifiers ie. bestSpeedCodec
    BailErr(err);

    DisposeHandle((Handle)imageDesc);
    imageDesc = NULL;

bail:
    return err;
}



static OSErr
Grabber_Close(GrabberPtr g)
{
    OSErr	err = noErr;

    if (g->seqGrab)
    {
        err = SGStop(g->seqGrab); BailErr(err);
        err = CDSequenceEnd(g->decomSeq);
        err = CloseComponent(g->seqGrab); BailErr(err);
    }

    DisposeGWorld(g->pGWorld);

    if (g->data != NULL)
        free(g->data);

    DisposePtr((Ptr)g);

bail:
    return err;
}



static OSErr
Grabber_Grab(GrabberPtr g)
{
    OSErr	err = noErr;

    g->grabDone = false;

    while (!g->grabDone && !err)
        err = SGIdle(g->seqGrab);

    return err;
}



InputVideoQT::InputVideoQT(char * name, Parameter * p):
        Module(p)
{
    g    = NULL;
    size_x  = GetIntValue(p, "size_x", 160);	// Quater-size
    size_y  = GetIntValue(p, "size_y", 120);

    if (size_x*size_y < 1)
    {
        Notify(msg_fatal_error, "Image size not supplied.\n");
        return;
    }

    Notify(msg_verbose, "Opening Grabber\n");

    if (Grabber_Open((GrabberPtr *)(&g), size_x, size_y) != noErr)
    {
        Notify(msg_fatal_error, "Sequence Grabber could not be created. Camera may not be connected.\n");
        return;
    }

    Notify(msg_verbose,"Opening Grabber - Done\n");

    AddOutput("INTENSITY", size_x, size_y);
    AddOutput("RED", size_x, size_y);
    AddOutput("GREEN", size_x, size_y);
    AddOutput("BLUE", size_x, size_y);
}



InputVideoQT::~InputVideoQT()
{
    Grabber_Close((GrabberPtr)g);
    Notify(msg_verbose, "Closing Grabber\n");
}



Module *
InputVideoQT::Create(char * name, Parameter * p)
{
    return new InputVideoQT(name, p);
}



void
InputVideoQT::Init()
{
    Notify(msg_warning, "InputQTAudio is not thread safe\n");

    intensity   = GetOutputArray("INTENSITY");
    red		= GetOutputArray("RED");
    green	= GetOutputArray("GREEN");
    blue		= GetOutputArray("BLUE");
}



void
InputVideoQT::Tick()
{
    Notify(msg_verbose, "Grabbing.\n");

    OSErr err = Grabber_Grab((GrabberPtr)g);
    if (err != noErr)
    {
        Notify(msg_fatal_error, "Sequence Grabber error.\n");
        return;
    }

    PixMapHandle	portPixMap = GetPortPixMap(((GrabberPtr)g)->pGWorld);

    BytePtr dataPixels = (BytePtr)(**portPixMap).baseAddr;
    int		w =  (**portPixMap).bounds.right - (**portPixMap).bounds.left;
    int		h = (**portPixMap).bounds.bottom - (**portPixMap).bounds.top;
    int		rb = GetPixRowBytes(portPixMap);

    long		ix = 0;
    for (int y=0; y<h; y++)
    {
        BytePtr p = dataPixels + y*rb + 1;
        for (int x=0; x<w; x++)
        {
            red[ix]		= (*p)/255.0; p++;
            green[ix]		= (*p)/255.0; p++;
            blue[ix]		= (*p)/255.0; p++;  p++;
            intensity[ix] 	= (red[ix] +green[ix] + blue[ix])/3.0;
            ix++;
        }
    }
}

#endif
