//
//    Arbiter.cc		This file is a part of the IKAROS project
//
//    Copyright (C) 2006 Christian Balkenius
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include "Arbiter.h"

using namespace ikaros;


Module * Arbiter::Create(char * name, Parameter * p)
{
    return new Arbiter(name, p);
}



Arbiter::Arbiter(char * name, Parameter * p):
        Module(p)
{
    input1		=	NULL;
    input2		=	NULL;
    value1		=	NULL;
    value2		=	NULL;
    output		=	NULL;
    value		=	NULL;
    size			=	0;

    AddInput("INPUT1");
    AddInput("INPUT2");
    AddInput("VALUE1");
    AddInput("VALUE2");
    AddOutput("OUTPUT");
    AddOutput("VALUE", 1);
}



void
Arbiter::SetSizes()
{
    int size1x = GetInputSizeX("INPUT1");
    int size1y = GetInputSizeY("INPUT1");

    int size2x = GetInputSizeX("INPUT2");
    int size2y = GetInputSizeY("INPUT2");

    if (size1x == unknown_size && size2x == unknown_size)
        return;

    if (size1x != unknown_size && size2x != unknown_size && (size1x != size2x || size1y != size2y))
    {
        Notify(msg_fatal_error, "Arbiter: The input sizes must be equal: INPUT1 = [%d, %d] INPUT2 = [%d, %d]\n", size1x, size1y, size2x, size2y);
        return;
    }

    if (size1x != unknown_size)
        SetOutputSize("OUTPUT", size1x, size1y);
    else	if (size2x != unknown_size)
        SetOutputSize("OUTPUT", size2x, size2y);
}



void
Arbiter::Init()
{
    input1		=	GetInputArray("INPUT1");
    input2		=	GetInputArray("INPUT2");
    value1		=	GetInputArray("VALUE1");
    value2		=	GetInputArray("VALUE2");
    output		=	GetOutputArray("OUTPUT");
    value		=	GetOutputArray("VALUE");
    size			=	GetOutputSize("OUTPUT");
}



Arbiter::~Arbiter()
{
}



void
Arbiter::Tick()
{
    if (value1[0] >= value2[0])
    {
        copy_array(output, input1, size);
        value[0] = value1[0];
    }
    else
    {
        copy_array(output, input2, size);
        value[0] = value2[0];
    }
}


