//
//	Differentiator.cc		This file is a part of the IKAROS project
//						A module to differentiate its input over time
//
//    Copyright (C) 2003  Christian Balkenius
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include "Differentiator.h"

Module *
Differentiator::Create(char * name, Parameter * p)
{
    return new Differentiator(name, p);
}



Differentiator::Differentiator(char * name, Parameter * p):
        Module(p)
{
    AddInput("INPUT");
    AddInput("RESET");
    AddOutput("OUTPUT");

    rectify			= GetIntValueFromList(p, "rectify", "no/yes");
    gain				= GetFloatValue(p, "gain", 1.0);

    reset_period		= GetIntValue(p, "reset_period", 1);
    reset_counter		= 0;

    reset			= NULL;
    input				= NULL;
    input_last			= NULL;
    output			= NULL;
}



void
Differentiator::SetSizes()
{
    int sx = GetInputSizeX("INPUT");
    int sy = GetInputSizeY("INPUT");
    if (sx != unknown_size && sy != unknown_size)
        SetOutputSize("OUTPUT", sx, sy);
}



void
Differentiator::Init()
{
    size_x	= GetInputSizeX("INPUT");
    size_y	= GetInputSizeY("INPUT");

    reset	= GetInputArray("RESET");
    input		= GetInputMatrix("INPUT");
    input_last	= create_matrix(size_x, size_y);

    output	= GetOutputMatrix("OUTPUT");
}



Differentiator::~Differentiator()
{
    destroy_matrix(input_last);
}



void
Differentiator::Tick()
{
    if (--reset_counter > 0)
        return;

    for (int i=0; i<size_x; i++)
        for (int j=0; j<size_y; j++)
            output[j][i] = gain*(input[j][i] - input_last[j][i]);

    for (int i=0; i<size_x; i++)
        for (int j=0; j<size_y; j++)
            input_last[j][i] = input[j][i];

    if (rectify)
        for (int i=0; i<size_x; i++)
            for (int j=0; j<size_y; j++)
                if (output[j][i] < 0)
                    output[j][i] = -output[j][i];

    if (reset != NULL && reset[0] > 0)
    {
        for (int i=0; i<size_x; i++)
            for (int j=0; j<size_y; j++)
            {
                input[j][i] = 0;
                input_last[j][i] = 0;
                output[j][i] = 0;
            }

        reset_counter = reset_period;
    }
}


