//
//		MatrixMultiply.cc	This file is a part of the IKAROS project
//						Implements a modules that calculates the matrix product of two inputs
//
//    Copyright (C) 2004 Christian Balkenius
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//	Created: 2004-03-22
//


#include "MatrixMultiply.h"

using namespace ikaros;

Module * MatrixMultiply::Create(char * name, Parameter * p)
{
    return new MatrixMultiply(name, p);
}



MatrixMultiply::MatrixMultiply(char * name, Parameter * p):
        Module(p)
{
    input1		=   NULL;
    input2		=   NULL;
    output		=   NULL;

    AddInput("INPUT1");
    AddInput("INPUT2");
    AddOutput("OUTPUT");
}



void
MatrixMultiply::Init()
{
    input1		=	GetInputMatrix("INPUT1");
    input2		=	GetInputMatrix("INPUT2");
    output		=	GetOutputMatrix("OUTPUT");

    size1_x		=	GetInputSizeX("INPUT1");
    size1_y		=	GetInputSizeY("INPUT1");
    size2_x		=	GetInputSizeX("INPUT2");
    size2_y		=	GetInputSizeY("INPUT2");
}



void MatrixMultiply::SetSizes()
{
    int size1_x = GetInputSizeX("INPUT1");
    int size1_y = GetInputSizeY("INPUT1");
    int size2_x = GetInputSizeX("INPUT2");
    int size2_y = GetInputSizeY("INPUT2");

    if (size1_x == unknown_size || size1_y == unknown_size || size2_x == unknown_size || size2_y == unknown_size)
        return;

    if (size1_x != size2_y)
    {
        Notify(msg_fatal_error, "MatrixMultiply: The sizes of the inputs are not compatible: INPUT1 = [%d, %d] INPUT2 = [%d, %d]\n", size1_x, size1_y, size2_x, size2_y);
        return;
    }

    SetOutputSize("OUTPUT", size2_x, size1_y);
}



MatrixMultiply::~MatrixMultiply()
{
}



void
MatrixMultiply::Tick()
{
    multiply(output, input1, input2, size2_x, size2_y, size1_x);
    /*
    	for(int j=0; j<size1_y; j++)
    		for(int i=0; i<size2_x; i++)
    		{
    			output[j][i] = 0;
    			for(int k = 0; k<size1_x; k++)
    				output[j][i]  += input1[j][k] * input2[k][i];
    		}
    */
}


