//
//	WebUI.cc		HTTP support for the IKAROS kernel
//
//    Copyright (C) 2005-2007  Christian Balkenius
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//    See http://www.ikaros-project.org/ for more information.
//
//    Created: August 31, 2005
//    Based on IKAROS_HTTP



#include "WebUI.h"

#ifdef USE_SOCKET

extern "C"
{
#include "jpeglib.h"
}

#ifdef WINDOWS
#include <direct.h>
#else
#include <unistd.h>
#endif

#include <cstring>
#include <fcntl.h>

using namespace ikaros;

//
//
// JPEG Compression Routines
//
//

static bool
SendColorJPEG(ServerSocket * socket, float ** r, float ** g, float ** b, int size_x, int size_y)
{
    if (r ==NULL) return false;
    if (g ==NULL) return false;
    if (b ==NULL) return false;

    bool clipping = false;

    JSAMPLE * image_buffer = new JSAMPLE [3*size_x];

    struct jpeg_compress_struct cinfo;
    struct jpeg_error_mgr jerr;

    JSAMPROW row_pointer[1];	/* pointer to JSAMPLE row[s] */
    int row_stride;				/* physical row width in image buffer */

    cinfo.err = jpeg_std_error(&jerr);

    jpeg_create_compress(&cinfo);	// Replace with ikaros error handler later

    FILE * outfile;
    if ((outfile = fopen("ikaros.tmp", "w+b")) == NULL)	// mktemp(tempname)
    {
        fprintf(stderr, "Could not create temporary file\n");
        return false;
    }

    jpeg_stdio_dest(&cinfo, outfile);

    // Set compression parameters

    cinfo.image_width = size_x; 	/* image width and height, in pixels */
    cinfo.image_height = size_y;
    cinfo.input_components = 3;		/* # of color components per pixel */
    cinfo.in_color_space = JCS_RGB; 	/* colorspace of input image */

    jpeg_set_defaults(&cinfo);
    jpeg_set_quality(&cinfo, 100, true);

    // Do the compression

    jpeg_start_compress(&cinfo, true);

    row_stride = size_x * 3;		/* JSAMPLEs per row in image_buffer */
    int j=0;

    while (cinfo.next_scanline < cinfo.image_height)
    {
        int x = 0;
        for (int i=0; i<size_x; i++)
        {
            float rr = (r[j][i] < 0.0 ? 0.0 : (r[j][i] > 1.0 ? 1.0 : r[j][i]));
            float gg = (g[j][i] < 0.0 ? 0.0 : (g[j][i] > 1.0 ? 1.0 : g[j][i]));
            float bb = (b[j][i] < 0.0 ? 0.0 : (b[j][i] > 1.0 ? 1.0 : b[j][i]));

            clipping |= (rr != r[j][i]) || (gg != g[j][i]) || (bb != b[j][i]);

            image_buffer[x++] = int(255.0*rr);
            image_buffer[x++] = int(255.0*gg);
            image_buffer[x++] = int(255.0*bb);
        }

        // Write to compressor
        row_pointer[0] = image_buffer; // & image_buffer[cinfo.next_scanline * row_stride];
        (void) jpeg_write_scanlines(&cinfo, row_pointer, 1);
        j++;
    }
    jpeg_finish_compress(&cinfo);
    jpeg_destroy_compress(&cinfo);

    // Send the file
    if (!socket->SendFile(NULL,"ikaros.tmp"))
        return false;

    fclose(outfile);

    delete [] image_buffer;

    return true;
}


int LUT_spectrum[256][3] =
    {
        {255,	0,	0},
        {255,	6,	0},
        {255,	12,	0},
        {255,	18,	0},
        {255,	24,	0},
        {255,	30,	0},
        {255,	36,	0},
        {255,	42,	0},
        {255,	48,	0},
        {255,	54,	0},
        {255,	60,	0},
        {255,	66,	0},
        {255,	72,	0},
        {255,	78,	0},
        {255,	84,	0},
        {255,	90,	0},
        {255,	96,	0},
        {255,	102,	0},
        {255,	108,	0},
        {255,	114,	0},
        {255,	120,	0},
        {255,	126,	0},
        {255,	132,	0},
        {255,	138,	0},
        {255,	144,	0},
        {255,	150,	0},
        {255,	156,	0},
        {255,	162,	0},
        {255,	168,	0},
        {255,	174,	0},
        {255,	180,	0},
        {255,	186,	0},
        {255,	192,	0},
        {255,	198,	0},
        {255,	204,	0},
        {255,	210,	0},
        {255,	216,	0},
        {255,	222,	0},
        {255,	228,	0},
        {255,	234,	0},
        {255,	240,	0},
        {255,	246,	0},
        {255,	252,	0},
        {252,	255,	0},
        {246,	255,	0},
        {240,	255,	0},
        {234,	255,	0},
        {228,	255,	0},
        {222,	255,	0},
        {216,	255,	0},
        {210,	255,	0},
        {204,	255,	0},
        {198,	255,	0},
        {192,	255,	0},
        {186,	255,	0},
        {180,	255,	0},
        {174,	255,	0},
        {168,	255,	0},
        {162,	255,	0},
        {156,	255,	0},
        {150,	255,	0},
        {144,	255,	0},
        {138,	255,	0},
        {132,	255,	0},
        {126,	255,	0},
        {120,	255,	0},
        {114,	255,	0},
        {108,	255,	0},
        {102,	255,	0},
        {96,	255,	0},
        {90,	255,	0},
        {84,	255,	0},
        {78,	255,	0},
        {72,	255,	0},
        {66,	255,	0},
        {60,	255,	0},
        {54,	255,	0},
        {48,	255,	0},
        {42,	255,	0},
        {36,	255,	0},
        {30,	255,	0},
        {24,	255,	0},
        {18,	255,	0},
        {12,	255,	0},
        {6,	255,	0},
        {0,	255,	0},
        {0,	255,	6},
        {0,	255,	12},
        {0,	255,	18},
        {0,	255,	24},
        {0,	255,	30},
        {0,	255,	36},
        {0,	255,	42},
        {0,	255,	48},
        {0,	255,	54},
        {0,	255,	60},
        {0,	255,	66},
        {0,	255,	72},
        {0,	255,	78},
        {0,	255,	84},
        {0,	255,	90},
        {0,	255,	96},
        {0,	255,	102},
        {0,	255,	108},
        {0,	255,	114},
        {0,	255,	120},
        {0,	255,	126},
        {0,	255,	132},
        {0,	255,	138},
        {0,	255,	144},
        {0,	255,	150},
        {0,	255,	156},
        {0,	255,	162},
        {0,	255,	168},
        {0,	255,	174},
        {0,	255,	180},
        {0,	255,	186},
        {0,	255,	192},
        {0,	255,	198},
        {0,	255,	204},
        {0,	255,	210},
        {0,	255,	216},
        {0,	255,	222},
        {0,	255,	228},
        {0,	255,	234},
        {0,	255,	240},
        {0,	255,	246},
        {0,	255,	252},
        {0,	252,	255},
        {0,	246,	255},
        {0,	240,	255},
        {0,	234,	255},
        {0,	228,	255},
        {0,	222,	255},
        {0,	216,	255},
        {0,	210,	255},
        {0,	204,	255},
        {0,	198,	255},
        {0,	192,	255},
        {0,	186,	255},
        {0,	180,	255},
        {0,	174,	255},
        {0,	168,	255},
        {0,	162,	255},
        {0,	156,	255},
        {0,	150,	255},
        {0,	144,	255},
        {0,	138,	255},
        {0,	132,	255},
        {0,	126,	255},
        {0,	120,	255},
        {0,	114,	255},
        {0,	108,	255},
        {0,	102,	255},
        {0,	96,	255},
        {0,	90,	255},
        {0,	84,	255},
        {0,	78,	255},
        {0,	72,	255},
        {0,	66,	255},
        {0,	60,	255},
        {0,	54,	255},
        {0,	48,	255},
        {0,	42,	255},
        {0,	36,	255},
        {0,	30,	255},
        {0,	24,	255},
        {0,	18,	255},
        {0,	12,	255},
        {0,	6,	255},
        {0,	0,	255},
        {6,	0,	255},
        {12,	0,	255},
        {18,	0,	255},
        {24,	0,	255},
        {30,	0,	255},
        {36,	0,	255},
        {42,	0,	255},
        {48,	0,	255},
        {54,	0,	255},
        {60,	0,	255},
        {66,	0,	255},
        {72,	0,	255},
        {78,	0,	255},
        {84,	0,	255},
        {90,	0,	255},
        {96,	0,	255},
        {102,	0,	255},
        {108,	0,	255},
        {114,	0,	255},
        {120,	0,	255},
        {126,	0,	255},
        {132,	0,	255},
        {138,	0,	255},
        {144,	0,	255},
        {150,	0,	255},
        {156,	0,	255},
        {162,	0,	255},
        {168,	0,	255},
        {174,	0,	255},
        {180,	0,	255},
        {186,	0,	255},
        {192,	0,	255},
        {198,	0,	255},
        {204,	0,	255},
        {210,	0,	255},
        {216,	0,	255},
        {222,	0,	255},
        {228,	0,	255},
        {234,	0,	255},
        {240,	0,	255},
        {246,	0,	255},
        {252,	0,	255},
        {255,	0,	252},
        {255,	0,	246},
        {255,	0,	240},
        {255,	0,	234},
        {255,	0,	228},
        {255,	0,	222},
        {255,	0,	216},
        {255,	0,	210},
        {255,	0,	204},
        {255,	0,	198},
        {255,	0,	192},
        {255,	0,	186},
        {255,	0,	180},
        {255,	0,	174},
        {255,	0,	168},
        {255,	0,	162},
        {255,	0,	156},
        {255,	0,	150},
        {255,	0,	144},
        {255,	0,	138},
        {255,	0,	132},
        {255,	0,	126},
        {255,	0,	120},
        {255,	0,	114},
        {255,	0,	108},
        {255,	0,	102},
        {255,	0,	96},
        {255,	0,	90},
        {255,	0,	84},
        {255,	0,	78},
        {255,	0,	72},
        {255,	0,	66},
        {255,	0,	60},
        {255,	0,	54},
        {255,	0,	48},
        {255,	0,	42},
        {255,	0,	36},
        {255,	0,	30},
        {255,	0,	24},
        {255,	0,	18},
        {255,	0,	12},
        {255,	0,	6},
        {255,	0,	0}
    };



static bool
SendSpectrumJPEG(ServerSocket * socket, float ** matrix, int size_x, int size_y)
{
    if (matrix ==NULL) return false;

    JSAMPLE * image_buffer = new JSAMPLE [3*size_x];

    struct jpeg_compress_struct cinfo;
    struct jpeg_error_mgr jerr;

    JSAMPROW row_pointer[1];	/* pointer to JSAMPLE row[s] */
    int row_stride;				/* physical row width in image buffer */

    cinfo.err = jpeg_std_error(&jerr);

    jpeg_create_compress(&cinfo);	// Replace with ikaros error ahdnler later

    FILE * outfile;
    if ((outfile = fopen("ikaros.tmp", "w+b")) == NULL)	// mktemp(tempname)
    {
        fprintf(stderr, "WebUI: Could not create temporary file\n");
        return false;
    }

    jpeg_stdio_dest(&cinfo, outfile);

    // Set compression parameters

    cinfo.image_width = size_x; 	/* image width and height, in pixels */
    cinfo.image_height = size_y;
    cinfo.input_components = 3;		/* # of color components per pixel */
    cinfo.in_color_space = JCS_RGB; 	/* colorspace of input image */

    jpeg_set_defaults(&cinfo);
    jpeg_set_quality(&cinfo, 100, true);

    // Do the compression

    jpeg_start_compress(&cinfo, true);
    row_stride = size_x * 3;		/* JSAMPLEs per row in image_buffer */
    int j=0;

    unsigned char * z = new unsigned char [size_x];
    while (cinfo.next_scanline < cinfo.image_height)
    {
        float_to_byte(z, matrix[j], 0.0, 1.0, size_x);	// allow specification of min and max later ***
        int x = 0;
        unsigned char * zz = z;

        for (int i=0; i<size_x; i++)
        {
            image_buffer[x++] = LUT_spectrum[*zz][0];
            image_buffer[x++] = LUT_spectrum[*zz][1];
            image_buffer[x++] = LUT_spectrum[*zz][2];
            zz++;
        }

        // Write to compressor
        row_pointer[0] = image_buffer; // & image_buffer[cinfo.next_scanline * row_stride];
        (void) jpeg_write_scanlines(&cinfo, row_pointer, 1);
        j++;
    }

    jpeg_finish_compress(&cinfo);
    jpeg_destroy_compress(&cinfo);

    // Send the file

    // Send the file
    if (!socket->SendFile(NULL,"ikaros.tmp"))
        return false;

    fclose(outfile);

    delete [] z;
    delete [] image_buffer;

    return true;
}



int LUT_fire[256][3] =
    {
        {0, 0, 31},
        {0, 0, 34},
        {0, 0, 38},
        {0, 0, 42},
        {0, 0, 46},
        {0, 0, 49},
        {0, 0, 53},
        {0, 0, 57},
        {0, 0, 61},
        {0, 0, 65},
        {0, 0, 69},
        {0, 0, 74},
        {0, 0, 78},
        {0, 0, 82},
        {0, 0, 87},
        {0, 0, 91},
        {1, 0, 96},
        {4, 0, 100},
        {7, 0, 104},
        {10, 0, 108},
        {13, 0, 113},
        {16, 0, 117},
        {19, 0, 121},
        {22, 0, 125},
        {25, 0, 130},
        {28, 0, 134},
        {31, 0, 138},
        {34, 0, 143},
        {37, 0, 147},
        {40, 0, 151},
        {43, 0, 156},
        {46, 0, 160},
        {49, 0, 165},
        {52, 0, 168},
        {55, 0, 171},
        {58, 0, 175},
        {61, 0, 178},
        {64, 0, 181},
        {67, 0, 185},
        {70, 0, 188},
        {73, 0, 192},
        {76, 0, 195},
        {79, 0, 199},
        {82, 0, 202},
        {85, 0, 206},
        {88, 0, 209},
        {91, 0, 213},
        {94, 0, 216},
        {98, 0, 220},
        {101, 0, 220},
        {104, 0, 221},
        {107, 0, 222},
        {110, 0, 223},
        {113, 0, 224},
        {116, 0, 225},
        {119, 0, 226},
        {122, 0, 227},
        {125, 0, 224},
        {128, 0, 222},
        {131, 0, 220},
        {134, 0, 218},
        {137, 0, 216},
        {140, 0, 214},
        {143, 0, 212},
        {146, 0, 210},
        {148, 0, 206},
        {150, 0, 202},
        {152, 0, 199},
        {154, 0, 195},
        {156, 0, 191},
        {158, 0, 188},
        {160, 0, 184},
        {162, 0, 181},
        {163, 0, 177},
        {164, 0, 173},
        {166, 0, 169},
        {167, 0, 166},
        {168, 0, 162},
        {170, 0, 158},
        {171, 0, 154},
        {173, 0, 151},
        {174, 0, 147},
        {175, 0, 143},
        {177, 0, 140},
        {178, 0, 136},
        {179, 0, 132},
        {181, 0, 129},
        {182, 0, 125},
        {184, 0, 122},
        {185, 0, 118},
        {186, 0, 114},
        {188, 0, 111},
        {189, 0, 107},
        {190, 0, 103},
        {192, 0, 100},
        {193, 0, 96},
        {195, 0, 93},
        {196, 1, 89},
        {198, 3, 85},
        {199, 5, 82},
        {201, 7, 78},
        {202, 8, 74},
        {204, 10, 71},
        {205, 12, 67},
        {207, 14, 64},
        {208, 16, 60},
        {209, 19, 56},
        {210, 21, 53},
        {212, 24, 49},
        {213, 27, 45},
        {214, 29, 42},
        {215, 32, 38},
        {217, 35, 35},
        {218, 37, 31},
        {220, 40, 27},
        {221, 43, 23},
        {223, 46, 20},
        {224, 48, 16},
        {226, 51, 12},
        {227, 54, 8},
        {229, 57, 5},
        {230, 59, 4},
        {231, 62, 3},
        {233, 65, 3},
        {234, 68, 2},
        {235, 70, 1},
        {237, 73, 1},
        {238, 76, 0},
        {240, 79, 0},
        {241, 81, 0},
        {243, 84, 0},
        {244, 87, 0},
        {246, 90, 0},
        {247, 92, 0},
        {249, 95, 0},
        {250, 98, 0},
        {252, 101, 0},
        {252, 103, 0},
        {252, 105, 0},
        {253, 107, 0},
        {253, 109, 0},
        {253, 111, 0},
        {254, 113, 0},
        {254, 115, 0},
        {255, 117, 0},
        {255, 119, 0},
        {255, 121, 0},
        {255, 123, 0},
        {255, 125, 0},
        {255, 127, 0},
        {255, 129, 0},
        {255, 131, 0},
        {255, 133, 0},
        {255, 134, 0},
        {255, 136, 0},
        {255, 138, 0},
        {255, 140, 0},
        {255, 141, 0},
        {255, 143, 0},
        {255, 145, 0},
        {255, 147, 0},
        {255, 148, 0},
        {255, 150, 0},
        {255, 152, 0},
        {255, 154, 0},
        {255, 155, 0},
        {255, 157, 0},
        {255, 159, 0},
        {255, 161, 0},
        {255, 162, 0},
        {255, 164, 0},
        {255, 166, 0},
        {255, 168, 0},
        {255, 169, 0},
        {255, 171, 0},
        {255, 173, 0},
        {255, 175, 0},
        {255, 176, 0},
        {255, 178, 0},
        {255, 180, 0},
        {255, 182, 0},
        {255, 184, 0},
        {255, 186, 0},
        {255, 188, 0},
        {255, 190, 0},
        {255, 191, 0},
        {255, 193, 0},
        {255, 195, 0},
        {255, 197, 0},
        {255, 199, 0},
        {255, 201, 0},
        {255, 203, 0},
        {255, 205, 0},
        {255, 206, 0},
        {255, 208, 0},
        {255, 210, 0},
        {255, 212, 0},
        {255, 213, 0},
        {255, 215, 0},
        {255, 217, 0},
        {255, 219, 0},
        {255, 220, 0},
        {255, 222, 0},
        {255, 224, 0},
        {255, 226, 0},
        {255, 228, 0},
        {255, 230, 0},
        {255, 232, 0},
        {255, 234, 0},
        {255, 235, 4},
        {255, 237, 8},
        {255, 239, 13},
        {255, 241, 17},
        {255, 242, 21},
        {255, 244, 26},
        {255, 246, 30},
        {255, 248, 35},
        {255, 248, 42},
        {255, 249, 50},
        {255, 250, 58},
        {255, 251, 66},
        {255, 252, 74},
        {255, 253, 82},
        {255, 254, 90},
        {255, 255, 98},
        {255, 255, 105},
        {255, 255, 113},
        {255, 255, 121},
        {255, 255, 129},
        {255, 255, 136},
        {255, 255, 144},
        {255, 255, 152},
        {255, 255, 160},
        {255, 255, 167},
        {255, 255, 175},
        {255, 255, 183},
        {255, 255, 191},
        {255, 255, 199},
        {255, 255, 207},
        {255, 255, 215},
        {255, 255, 223},
        {255, 255, 227},
        {255, 255, 231},
        {255, 255, 235},
        {255, 255, 239},
        {255, 255, 243},
        {255, 255, 247},
        {255, 255, 251},
        {255, 255, 255},
        {255, 255, 255},
        {255, 255, 255},
        {255, 255, 255},
        {255, 255, 255},
        {255, 255, 255},
        {255, 255, 255},
        {255, 255, 255}
    };



static bool
SendFireJPEG(ServerSocket * socket, float ** matrix, int size_x, int size_y)
{
    if (matrix ==NULL) return false;

    JSAMPLE * image_buffer = new JSAMPLE [3*size_x];

    struct jpeg_compress_struct cinfo;
    struct jpeg_error_mgr jerr;

    JSAMPROW row_pointer[1];	/* pointer to JSAMPLE row[s] */
    int row_stride;				/* physical row width in image buffer */

    cinfo.err = jpeg_std_error(&jerr);

    jpeg_create_compress(&cinfo);	// Replace with ikaros error ahdnler later

    FILE * outfile;
    if ((outfile = fopen("ikaros.tmp", "w+b")) == NULL)	// mktemp(tempname)
    {
        fprintf(stderr, "WebUI: Could not create temporary file\n");
        return false;
    }

    jpeg_stdio_dest(&cinfo, outfile);

    // Set compression parameters

    cinfo.image_width = size_x; 	/* image width and height, in pixels */
    cinfo.image_height = size_y;
    cinfo.input_components = 3;		/* # of color components per pixel */
    cinfo.in_color_space = JCS_RGB; 	/* colorspace of input image */

    jpeg_set_defaults(&cinfo);
    jpeg_set_quality(&cinfo, 100, true);

    // Do the compression

    jpeg_start_compress(&cinfo, true);
    row_stride = size_x * 3;		/* JSAMPLEs per row in image_buffer */
    int j=0;

    unsigned char * z = new unsigned char [size_x];
    while (cinfo.next_scanline < cinfo.image_height)
    {
        float_to_byte(z, matrix[j], 0.0, 1.0, size_x);	// allow specification of min and max later ***
        int x = 0;
        unsigned char * zz = z;

        for (int i=0; i<size_x; i++)
        {
            image_buffer[x++] = LUT_fire[*zz][0];
            image_buffer[x++] = LUT_fire[*zz][1];
            image_buffer[x++] = LUT_fire[*zz][2];
            zz++;
        }

        // Write to compressor
        row_pointer[0] = image_buffer; // & image_buffer[cinfo.next_scanline * row_stride];
        (void) jpeg_write_scanlines(&cinfo, row_pointer, 1);
        j++;
    }

    jpeg_finish_compress(&cinfo);
    jpeg_destroy_compress(&cinfo);

    // Send the file

    // Send the file
    if (!socket->SendFile(NULL,"ikaros.tmp"))
        return false;

    fclose(outfile);

    delete [] z;
    delete [] image_buffer;

    return true;
}



static bool
SendGreenJPEG(ServerSocket * socket, float ** matrix, int size_x, int size_y)
{
    if (matrix ==NULL) return false;

    JSAMPLE * image_buffer = new JSAMPLE [3*size_x];

    struct jpeg_compress_struct cinfo;
    struct jpeg_error_mgr jerr;

    JSAMPROW row_pointer[1];	/* pointer to JSAMPLE row[s] */
    int row_stride;				/* physical row width in image buffer */

    cinfo.err = jpeg_std_error(&jerr);

    jpeg_create_compress(&cinfo);	// Replace with ikaros error ahdnler later

    FILE * outfile;
    if ((outfile = fopen("ikaros.tmp", "w+b")) == NULL)	// mktemp(tempname)
    {
        fprintf(stderr, "WebUI: Could not create temporary file\n");
        return false;
    }

    jpeg_stdio_dest(&cinfo, outfile);

    // Set compression parameters

    cinfo.image_width = size_x; 	/* image width and height, in pixels */
    cinfo.image_height = size_y;
    cinfo.input_components = 3;		/* # of color components per pixel */
    cinfo.in_color_space = JCS_RGB; 	/* colorspace of input image */

    jpeg_set_defaults(&cinfo);
    jpeg_set_quality(&cinfo, 100, true);

    // Do the compression

    jpeg_start_compress(&cinfo, true);
    row_stride = size_x * 3;		/* JSAMPLEs per row in image_buffer */
    int j=0;

    unsigned char * z = new unsigned char [size_x];
    while (cinfo.next_scanline < cinfo.image_height)
    {
        float_to_byte(z, matrix[j], 0.0, 1.0, size_x);	// allow specification of min and max later ***
        int x = 0;
        unsigned char * zz = z;

        for (int i=0; i<size_x; i++)
        {
            image_buffer[x++] = 0;
            image_buffer[x++] = *zz;
            image_buffer[x++] = 0;
            zz++;
        }

        // Write to compressor
        row_pointer[0] = image_buffer; // & image_buffer[cinfo.next_scanline * row_stride];
        (void) jpeg_write_scanlines(&cinfo, row_pointer, 1);
        j++;
    }

    jpeg_finish_compress(&cinfo);
    jpeg_destroy_compress(&cinfo);

    // Send the file
    if (!socket->SendFile(NULL,"ikaros.tmp"))
        return false;

    fclose(outfile);

    delete [] z;
    delete [] image_buffer;

    return true;
}



static bool
SendGrayJPEG(ServerSocket * socket, float ** matrix, int sizex, int sizey)	// Compress image to jpg and send using temporary file
{
    if (matrix == NULL) return false;

    float maximum, minimum;
    minmax(minimum, maximum, matrix, sizex, sizey);

    // Write the image

    JSAMPLE * image_buffer = new JSAMPLE [sizex];

    struct jpeg_compress_struct cinfo;
    struct jpeg_error_mgr jerr;

    FILE * outfile;
    JSAMPROW row_pointer[1];	/* pointer to JSAMPLE row[s] */
    int row_stride;				/* physical row width in image buffer */

    cinfo.err = jpeg_std_error(&jerr);

    jpeg_create_compress(&cinfo);	// Replace with ikaros error handler later

    if ((outfile = fopen("ikaros.tmp", "w+b")) == NULL)	// mktemp(tempname)
    {
        fprintf(stderr, "Could no create temporary file\n");
        return false;
    }

    jpeg_stdio_dest(&cinfo, outfile);

    // Set compression parameters

    cinfo.image_width = sizex; 	/* image width and height, in pixels */
    cinfo.image_height = sizey;
    cinfo.input_components = 1;	/* # of color components per pixel */
    cinfo.in_color_space = JCS_GRAYSCALE;

    jpeg_set_defaults(&cinfo);
    jpeg_set_quality(&cinfo, 100, true);	// Highest quality

    // Do the compression

    jpeg_start_compress(&cinfo, true);

    row_stride = sizex * 1;	/* JSAMPLEs per row in image_buffer */
    int j=0;

    while (cinfo.next_scanline < cinfo.image_height)
    {
        // Convert row to image buffer (assume max == 1 for now)

        JSAMPLE * ib = image_buffer;
        if (maximum != minimum)
            float_to_byte(image_buffer, matrix[j], minimum, maximum, sizex);
        else
            for (int i=0; i<sizex; i++)
                *ib++ = 0;

        // Write to compressor
        row_pointer[0] = image_buffer; // & image_buffer[cinfo.next_scanline * row_stride];
        (void) jpeg_write_scanlines(&cinfo, row_pointer, 1);
        j++;
    }

    jpeg_finish_compress(&cinfo);
    jpeg_destroy_compress(&cinfo);

    // Send the file
    if (!socket->SendFile(NULL,"ikaros.tmp"))
    {
        fclose(outfile);
        delete [] image_buffer;
        return false;
    }

    fclose(outfile);
    delete [] image_buffer;
    return true;
}



static bool
SendTextData(ServerSocket * socket, float ** matrix, int sizex, int sizey)
{
    for (int j=0; j<sizey; j++)
        for (int i=0; i<sizex; i++)
            if (i==0 && j==0)
                socket->Send("%.4f", matrix[0][0]);	// no space before first
            else
                socket->Send(" %.4f", matrix[j][i]);

    return true;
}


static bool
SendJSONMatrixData(ServerSocket * socket, char * module, char * source, float ** matrix, int sizex, int sizey)
{
    if (matrix == NULL)
        return false;

    socket->Send("\t%s:\n\t[\n", source);	// module
    for (int j=0; j<sizey; j++)
    {
        socket->Send("\t\t[%.4f", matrix[j][0]);
        for (int i=1; i<sizex; i++)
            socket->Send(", %.4f", matrix[j][i]);
        if (j<sizey-1)
            socket->Send("],\n");
        else
            socket->Send("]\n\t]");
    }

    return true;
}


DataSource::DataSource(char * s, Module_IO * io, DataSource * n)
{
    next = n;
    name = s;
    module_io = io;
}



DataSource::~DataSource()
{
    delete next;
}



ModuleData::ModuleData(char * module_name, Module * m, ModuleData * n)
{
    name = module_name;
    module = m;
    next = n;
    source = NULL;
}



ModuleData::~ModuleData()
{
    delete next;
    delete source;
}



void
ModuleData::AddSource(char * s, Module_IO * io)
{
    for (DataSource * sd=source; sd != NULL; sd=sd->next)
        if (!strcmp(sd->name, s))
            return;

    source = new DataSource(s, io, source);
}



void
WebUI::AddDataSource(char * module, char * source)
{
//	printf("AddDataSource: %s.%s\n", module, source);

    Module * m;
    Module_IO * io;
    XMLElement * group = k->xmlDoc->xml->FindElement("group");

    // Legacy support

    if (group == NULL)
    {
        group = k->xmlDoc->xml->FindElement("network");
        if (group != NULL)
            group = group->content->FindElement("modules");
    }

    if (group == NULL)
    {
        printf("WebUI ERROR: Could not find <group> or <modules> element.\n");
        return;
    }

    if (!k->GetSource(group, m, io, module, source))
    {
        printf("WebUI ERROR: Could not add data source %s.%s (not found)\n", module, source);
        return;
    }

    for (ModuleData * md=view_data; md != NULL; md=md->next)
        if (!strcmp(md->name, module))
        {
            md->AddSource(source, io);
            return;
        }

    view_data = new ModuleData(module, m, view_data);
    view_data->AddSource(source, io);
}



void
WebUI::SendAllJSONData()
{
    socket->Send("{\n");
    socket->Send("state: %d,\n", ui_state);
    socket->Send("iteration: %d", k->GetTick());

    if (view_data != NULL)
        socket->Send(",\n");
    else
        socket->Send("\n");

    for (ModuleData * md=view_data; md != NULL; md=md->next)
    {
        socket->Send("%s:\n{\n", md->name);
        for (DataSource * sd=md->source; sd != NULL; sd=sd->next)
        {
            SendJSONMatrixData(socket, md->name, sd->name, sd->module_io->matrix[0], sd->module_io->sizex, sd->module_io->sizey);
            if (sd->next != NULL)
                socket->Send(",\n");
            else
                socket->Send("\n");
        }

        if (md->next != NULL)
            socket->Send("},\n");
        else
            socket->Send("}\n");
    }

    socket->Send("}\n");
}



void
WebUI::SendMinimalJSONData()
{
    socket->Send("{\n");
    socket->Send("state: %d,\n", ui_state);
    socket->Send("iteration: %d", k->GetTick());
    socket->Send("}\n");
}



WebUI::WebUI(Kernel * kernel)
{
    k = kernel;
    webui_dir = NULL;
    xml = NULL;
    ui_state = ui_state_pause;
    view_data = NULL;
    debug_mode = false;
    isRunning = false;

    if (k->options->GetOption('w'))
    {
        port = string_to_int(k->options->GetArgument('w'), PORT);
        k->Notify(msg_verbose, "Setting up WebUI port at %d\n", port);
    }

    if (k->options->GetOption('W'))
    {
        port = string_to_int(k->options->GetArgument('W'), PORT);
        k->Notify(msg_verbose, "Setting up WebUI port at %d in debug mode\n", port);
        debug_mode = true;
    }

    k->Notify(msg_print, "Starting Ikaros WebUI server.\n");
    k->Notify(msg_print, "Connect from a browser on this computer with the URL \"http://127.0.0.1:%d/\".\n", port);
    k->Notify(msg_print, "Use the URL \"http://<servername>:%d/\" from other computers.\n", port);

    webui_dir = create_formatted_string("%s%s", k->ikaros_dir, WEBUIPATH);    

//    printf("WebUI Path: %s\n", webui_dir);
	
//    char * p = create_formatted_string("%s%s", webui_dir, "/log.tmp");
//    k->logfile = fopen(p, "wb+");
//    destroy_string(p);
    
    ReadXML(k->xmlDoc);

    socket =  new ServerSocket(port);
}



WebUI::~WebUI()
{
    delete socket;
}



// Layout constants

const float margin_x = 14.5;
const float margin_y = 80.5;

const float padding_x = 14.0;
const float padding_y = 14.0;

const float object_spacing = 15.0;
const float object_size = 140.0;	// was 70 / 100
const float title_height = 20.0;

const float offset_x = margin_x+padding_x;
const float offset_y = margin_y+padding_y+title_height;

const float menu_offset = 15.0;
const float	menu_width = 150.0;

const float view_min_width = 700;
const float view_min_height = 500;

void
WebUI::SendMenu(float x, float y, char * name)
{
    int v = 0;
    for (XMLElement * xml_view = xml->GetElement("view"); xml_view != NULL; xml_view = xml_view->next->FindElement("view"), v++)
    {
        char buf[32] = "";

        char * n = xml_view->GetAttribute("name");

        if (n == NULL)
        {
            sprintf(buf, "view%d", v);
            n = buf;
        }

        socket->Send("<a xlink:href='%s.svg'>", n);

        if (equal_strings(n, name))
            socket->Send("<rect x='%.2f' y='%.2f' width='150' height='20' rx='10' ry='10' fill='#404040' stroke='none' />\n", x, y+25*v);
        else
        {
            socket->Send("<rect x='%.2f' y='%.2f' width='150' height='20' rx='10' ry='10' fill='#1C1C1C' />\n", x, y+25*v);
            socket->Send("<circle cx='%.2f' cy='%.2f' r='7' fill='#888888' />", x+139, y+25*v+10);
            float a = x+139;
            float b = y+25*v+10;
            socket->Send("<polygon points='%.1f,%.1f %.1f,%.1f %.1f,%.1f %.1f,%.1f %.1f,%.1f %.1f,%.1f %.1f,%.1f' />",
                         a+4.5, b,
                         a+0.0, b-4.0,
                         a+0.0, b-1.0,
                         a-4.0, b-1.0,
                         a-4.0, b+1.0,
                         a+0.0, b+1.0,
                         a+0.0, b+4.0
                        );
        }

        socket->Send("<text x='%.2f' y='%.2f' style='font-size:12px;font-family:sans-serif;fill:#BABABA;'>%s</text>\n", x+15, y+25*v+15, n);
        socket->Send("</a>");
    }
}



void
WebUI::Send404()
{

    if (debug_mode)
        printf("Sending 404.html\n");

    // Send Header

    socket->Send("HTTP/1.1 200 OK\n");
    socket->Send("Content-Type: image/svg+xml\n");
    socket->Send("\n");

    float view_width_px = view_min_width;
    float view_height_px = view_min_height;

    // Send SVG Header

    socket->Send("<?xml version='1.0' encoding='iso-8859-1' standalone='no'?>\n");
//			socket->Send("<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\"  \"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">\n");
//			socket->Send("<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 20001102//EN\"  \"http://www.w3.org/TR/2000/CR-SVG-20001102/DTD/svg-20001102.dtd\">\n");
//          socket->Send("<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 20010904//EN\" \"http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd\">\n"); // This doctype will keep Adobe SVG 3.0 plugin and Safari happy
    socket->Send("<svg\n");
    socket->Send("   width='%.2f'\n", 2*margin_x + view_width_px);
    socket->Send("   height='%.2f'\n", margin_y + margin_x + view_height_px);
    socket->Send("   version='1.0'\n");
    socket->Send("   xmlns='http://www.w3.org/2000/svg'\n");
    socket->Send("   xmlns:xlink='http://www.w3.org/1999/xlink'\n");
    socket->Send(">\n\n");
    socket->Send("<title>Ikaros Viewer</title>\n\n");

    socket->Send("<rect x='0' y='0' width='%.2f' height='%.2f' fill='black' />n\n", 2*margin_x + view_width_px, margin_y + margin_x + view_height_px);

    socket->Send("<rect x='%.2f' y='%.2f' width='%.2f' height='%.2f' rx='5' ry='5' stroke='#666666' fill='#303030' />n\n", margin_x, margin_y, view_width_px, view_height_px);
    socket->Send("<text x='40' y='110' fill='white'>This view can not be found.</text>\n");
    socket->Send("<text x='40' y='130' fill='white'>Select from the menu to the right.</text>\n");

    socket->SendFile("", "viewer.svg");
    //socket->Send("<script type='text/javascript'>\n");
    socket->Send("<script type='text/ecmascript'>\n");
    socket->Send("<![CDATA[\n");
//    socket->Send("top.resizeBy(%.0f-innerWidth,%.0f-innerHeight);", 2*margin_x + view_width_px, margin_y + margin_x + view_height_px);
    socket->Send("]]>\n");
    socket->Send("</script>\n");

    SendMenu(margin_x + view_width_px-menu_width-padding_x, margin_y+padding_y, "");
    socket->Send("</svg>\n");
}



void
WebUI::SendView(char * view)
{
    // Replace %20 and others with a space

    int j = 0;
    for (int i=0; view[i]; i++)
        if (view[j] != '%')
            view[i] = view[j++];
        else
        {
            view[i] = ' ';
            j+=3;
        }

    if (debug_mode)
        printf("Sending View: %s\n", view);

    delete view_data;
    view_data = NULL;

    int v = 0;

    for (XMLElement * xml_view = xml->GetElement("view"); xml_view != NULL; xml_view = xml_view->next->FindElement("view"), v++)
    {
        char * n = xml_view->GetAttribute("name");
        if (
            (n != NULL && !strncmp(&view[1], n, strlen(view)-5)) ||	// test view name
            (!strncmp(view, "/view", 5) && '0'+v == view[5])		// test view number 0-9 for backward compatibility
        )
        {
            // Calculate View Size

            int view_width = 0;
            int view_height = 0;

            for (XMLElement * xml_uiobject = xml_view->GetElement("object"); xml_uiobject != NULL; xml_uiobject = xml_uiobject->next->FindElement("object"))
            {
                // char * object_class = xml_uiobject->FindAttribute("class");
                int x = string_to_int(xml_uiobject->GetAttribute("x"), -1);
                int y = string_to_int(xml_uiobject->GetAttribute("y"), -1);
                int width = string_to_int(xml_uiobject->GetAttribute("w"), 1);
                int height = string_to_int(xml_uiobject->GetAttribute("h"), 1);

                if (x+width > view_width)
                    view_width = x+width;

                if (y+height > view_height)
                    view_height = y+height;
            }

            float view_width_px = 2*padding_x + view_width*object_size +(view_width-1)*object_spacing + menu_offset + menu_width;
            float view_height_px = 2*padding_y + view_height*(object_size+title_height)+(view_height-1)*object_spacing;

            // set minimum view size here if larger than what is set above (from <view size=.../>)

            view_width_px = max(view_min_width, view_width_px);
            view_height_px = max(view_min_height, view_height_px);

            // Send Header - Since the HTTP header somehow makes Safari download instead of displaying the SVG, it is not included

            socket->Send("HTTP/1.1 200 OK\n");
            socket->Send("Content-Type: image/svg+xml\n");
            socket->Send("\n");

            // Send SVG Header

            socket->Send("<?xml version='1.0' encoding='iso-8859-1' standalone='no'?>\n");
//			socket->Send("<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\"  \"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">\n");
//			socket->Send("<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 20001102//EN\"  \"http://www.w3.org/TR/2000/CR-SVG-20001102/DTD/svg-20001102.dtd\">\n");
//          socket->Send("<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 20010904//EN\" \"http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd\">\n"); // This doctype will keep Adobe SVG 3.0 plugin and Safari happy
            socket->Send("<svg\n");
            socket->Send("   width='%.2f'\n", 2*margin_x + view_width_px);
            socket->Send("   height='%.2f'\n", margin_y + margin_x + view_height_px);
            socket->Send("   version='1.0'\n");
            socket->Send("   xmlns='http://www.w3.org/2000/svg'\n");
            socket->Send("   xmlns:xlink='http://www.w3.org/1999/xlink'\n");
            socket->Send("   style='background-color: black'\n"); // *** probably not correct but looks better ***
            socket->Send(">\n\n");
            socket->Send("<title>Ikaros Viewer</title>\n\n");

            socket->Send("<rect x='0' y='0' width='%.2f' height='%.2f' fill='black' />n\n", 2*margin_x + view_width_px, margin_y + margin_x + view_height_px);

            socket->Send("<rect x='%.2f' y='%.2f' width='%.2f' height='%.2f' rx='5' ry='5' stroke='#666666' fill='#303030' />n\n", margin_x, margin_y, view_width_px, view_height_px);

            char * a;
            XMLElement * e = k->xmlDoc->xml->FindElement("group");	// *** check WebUI xmlDoc ***
            if (e != NULL)
            {
                a = e->GetAttribute("title");
                if (a)
                    socket->Send("<text x='%.2f' y='%.2f' fill='#FFFFFF' text-anchor='end' style='font-size:30px;font-family:sans-serif;fill:#BABABA;'>%s</text>\n", view_width_px, margin_y-25, a);
            }

            if (!socket->SendFile("", "viewer.svg"))
                printf("Cannot find \"viewer.svg\"; webui_dir=\"%s\"\n", webui_dir);

//          socket->Send("<script type='text/javascript'>\n");
            socket->Send("<script type='text/ecmascript'>\n");
            socket->Send("<![CDATA[\n");

            // Send the UIObjects

            for (XMLElement * xml_uiobject = xml_view->GetElement("object"); xml_uiobject != NULL; xml_uiobject = xml_uiobject->next->FindElement("object"))
            {
                char * object_class = xml_uiobject->GetAttribute("class");
                int x = string_to_int(xml_uiobject->GetAttribute("x"), 0);
                int y = string_to_int(xml_uiobject->GetAttribute("y"), 0);
                int width = string_to_int(xml_uiobject->GetAttribute("w"), 1);
                int height = string_to_int(xml_uiobject->GetAttribute("h"), 1);

                socket->Send("add(new %s({", object_class);
                for (XMLAttribute * p = xml_uiobject->attributes; p != NULL; p = (XMLAttribute *)(p->next))
                    if (!(!strcmp(p->name, "x") || !strcmp(p->name, "y") || !strcmp(p->name, "w") || !strcmp(p->name, "h") || !strcmp(p->name, "class")))
                    {
                        if ('0' <= p->value[0] && p->value[0] <='9' || p->value[0] == '-')
                            socket->Send("%s:%s, ", p->name, p->value);
                        else if (p->value[0] == '[')
                            socket->Send("%s:%s, ", p->name, p->value);	// arrays are sent verbatim
                        else
                            socket->Send("%s:'%s', ", p->name, p->value); // quote other content
                    }
                socket->Send("x:%.2f, y:%.2f, width:%.2f, height:%.2f}));\n",
                             offset_x+(object_size+object_spacing)*x, offset_y+(title_height+object_size+object_spacing)*y,
                             object_size+(object_size+object_spacing)*float(width-1), object_size+(title_height+object_size+object_spacing)*float(height-1));
            }

            // Send automatic window resize

//          socket->Send("top.resizeBy(%.0f-innerWidth,%.0f-innerHeight);", 2*margin_x + view_width_px, margin_y + margin_x + view_height_px);

            // Send Footer

            socket->Send("]]>\n");
            socket->Send("</script>\n");

            SendMenu(margin_x + view_width_px-menu_width-padding_x, margin_y+padding_y, n);
            
            // Send command to update on changed view
            
            socket->Send("<script type='text/ecmascript'>\n");
            socket->Send("<![CDATA[\n");
            socket->Send("update();\n");
            socket->Send("]]>\n");
            socket->Send("</script>\n");


            socket->Send("</svg>\n");
            return;
        }
    }

    Send404();
}



void
WebUI::Run()
{
    if (socket == NULL)
        return; // ERROR
    chdir(webui_dir);
    k->timer->Restart();
    tick = 0;

    while (!k->Terminate())
    {
        if (socket->GetRequest())
        {
            if (socket->TestArgument(0, "GET"))
                HandleHTTPRequest();
            socket->Close();
        }
        else if (!isRunning)
        {
            Timer::Sleep(10);	// Wait 10ms to avoid wasting cycles if there are no requests
        }

        if (isRunning)
        {
            chdir(k->ikc_dir);
            k->Tick();
            tick++;
            chdir(webui_dir);

            if (k->tick_length > 0)
            {
                float lag = k->timer->WaitUntil(float(tick*k->tick_length));
                if (lag > 0.1) k->Notify(msg_warning, "Lagging %.2f ms at tick = %ld\n", lag, k->tick);
            }
        }
    }
}



void
WebUI::HandleHTTPRequest()
{
    if (debug_mode)
        printf("HTTP Request: %s %s\n", socket->GetArgument(0), socket->GetArgument(1));

    if (socket->TestArgument(1, "/stop"))
    {
        ui_state = ui_state_stop;
        SendAllJSONData();
        k->Notify(msg_terminate, "Sent by WebUI.\n");
    }

    else if (socket->TestArgument(1, "/step"))
    {
        ui_state = ui_state_pause;
        chdir(k->ikc_dir);
        k->Tick();
        chdir(webui_dir);
        SendAllJSONData();
        isRunning = false;
    }

    else if (socket->TestArgument(1, "/runstep"))
    {
        ui_state = ui_state_run;
        chdir(k->ikc_dir);
        k->Tick();
        chdir(webui_dir);
        SendAllJSONData();
        isRunning = false;
    }

    else if (socket->TestArgument(1, "/faststep"))
    {
        ui_state = ui_state_fast;
        chdir(k->ikc_dir);
        k->Tick();
        chdir(webui_dir);
        SendMinimalJSONData();
        isRunning = false;
    }

    else if (socket->TestArgument(1, "/astep")) // asynchronous step - send data for last iteration - sometimes faster
    {
        ui_state = ui_state_pause;
        SendAllJSONData();
        chdir(k->ikc_dir);
        k->Tick();
        chdir(webui_dir);
        isRunning = false;
    }

    else if (socket->TestArgument(1, "/update"))
    {
        SendAllJSONData();
    }

    else if (socket->TestArgument(1, "/pause"))
    {
        ui_state = ui_state_pause;
        SendAllJSONData();
        isRunning = false;
    }

    else if (socket->TestArgument(1, "/free"))
    {
        ui_state = ui_state_free;
        socket->Send("FREE\n");
        k->timer->Restart();
        tick = 0;
        isRunning = true;
    }

    else if (socket->TestPath(1, "/uses"))
    {
        char * module = new char [256];
        char * output = new char [256];
        int c = sscanf(socket->GetArgument(1), "/uses/%[^/]/%[^/]", module, output);
        if (c == 2)
            AddDataSource(module, output);
        socket->Send("OK\n");
    }

    else if (socket->TestPath(1, "/getlog"))
    {
        if (k->logfile)
            socket->SendFile(NULL, "logfile");
    }

    else if (socket->TestArgumentExtension(1, ".svg"))
        SendView(socket->GetArgument(1));

    else if (socket->TestPath(1, "/module/"))
    {
        char module[256], output[256], img[256];
        int c = sscanf(socket->GetArgument(1), "/module/%[^/]/%[^/]/%[^/]", module, output, img);

        char m1[256], m2[256], m3[256];
        int cc2= sscanf(module, "%[^+]+%[^+]+%[^+]", m1, m2, m3);

        if (cc2 == 1) // use same module for all sources if only one name is given
        {
            strcpy(m2, m1);
            strcpy(m3, m1);
            cc2 = 3;
        }

        if (c == 3 && cc2 == 3) // Color Image Request
        {
            Module * mod1 = k->GetModule(m1);
            if (mod1 == NULL) return;

            Module * mod2 = k->GetModule(m2);
            if (mod2 == NULL) return;

            Module * mod3 = k->GetModule(m3);
            if (mod3 == NULL) return;

            if (c == 3 && !strcmp(img, "rgb.jpg"))
            {
                bool err = false;
                char red[256], green[256], blue[256];
                int cc = sscanf(output, "%[^+]+%[^+]+%[^+]", red, green, blue);

                if (cc != 3) err = true;	// Error

                int sx = mod1->GetOutputSizeX(red);
                int sy = mod1->GetOutputSizeY(red);

                int sx2 = mod2->GetOutputSizeX(green);
                int sy2= mod2->GetOutputSizeY(green);

                if (sx != sx2) err = true;	// ERROR
                if (sy != sy2) err = true;	// ERROR

                int sx3 = mod3->GetOutputSizeX(blue);
                int sy3 = mod3->GetOutputSizeY(blue);

                if (sx != sx3) err = true;	// ERROR
                if (sy != sy3) err = true;	// ERROR

                if (!err)
                    SendColorJPEG(socket, mod1->GetOutputMatrix(red), mod2->GetOutputMatrix(green), mod3->GetOutputMatrix(blue), sx, sy);
                else
                    socket->SendFile("", "404.jpg");
            }
        }

        Module * m = k->GetModule(module);
        if (m == NULL) return;

        if (c == 3 && !strcmp(img, "gray.jpg"))
        {
            int sx = m->GetOutputSizeX(output);
            int sy = m->GetOutputSizeY(output);
            if (!SendGrayJPEG(socket, m->GetOutputMatrix(output), sx, sy))
                socket->SendFile("", "no_image.jpg");
        }

        else if (c == 3 && !strcmp(img, "green.jpg"))
        {
            int sx = m->GetOutputSizeX(output);
            int sy = m->GetOutputSizeY(output);
            if (!SendGreenJPEG(socket, m->GetOutputMatrix(output), sx, sy))
                socket->SendFile("", "404.jpg");

        }

        else if (c == 3 && !strcmp(img, "spectrum.jpg"))
        {
            int sx = m->GetOutputSizeX(output);
            int sy = m->GetOutputSizeY(output);
            if (!SendSpectrumJPEG(socket, m->GetOutputMatrix(output), sx, sy))
                socket->SendFile("", "404.jpg");
        }

        else if (c == 3 && !strcmp(img, "fire.jpg"))
        {
            int sx = m->GetOutputSizeX(output);
            int sy = m->GetOutputSizeY(output);
            if (!SendFireJPEG(socket, m->GetOutputMatrix(output), sx, sy))
                socket->SendFile("", "404.jpg");
        }

        else if (c == 3 && !strcmp(img, "data.txt"))
        {
            int sx = m->GetOutputSizeX(output);
            int sy = m->GetOutputSizeY(output);
            if (!SendTextData(socket, m->GetOutputMatrix(output), sx, sy))
                socket->SendFile("", "404.jpg");
        }
    }

    else if (
        socket->TestArgumentExtension(1, ".xml") ||
        socket->TestArgumentExtension(1, ".jpg") ||
        socket->TestArgumentExtension(1, ".html") ||
        socket->TestArgumentExtension(1, ".css") ||
        socket->TestArgumentExtension(1, ".png") ||
        socket->TestArgumentExtension(1, ".svg") ||
        socket->TestArgumentExtension(1, ".js") ||
        socket->TestArgumentExtension(1, ".gif") ||
        socket->TestArgumentExtension(1, ".ico"))
    {
        if (!socket->SendFile("", &(socket->GetArgument(1)[1])))
        {
            // Send 404 if available

            if (socket->TestArgumentExtension(1, ".jpg"))
                socket->SendFile("", "404.jpg");
            else if (socket->TestArgumentExtension(1, ".html"))
                socket->SendFile("", "404.html");
        }
    }

    else if (socket->TestPath(1, "/"))
    {
        int v = 0;
        if (!xml || xml->GetElement("view") == NULL)
        {
            socket->Send("<html>\n<head>\n<title>Ikaros Web View</title>\n</head>\n<body>\n<h1>Ikaros Web View</h1>\n<p>No views were defined.</p></ul>\n</body>\n</html>");
            return;
        }

        // Send index page

        socket->Send("<html>\n<head>\n<title>Ikaros Web View</title>\n</head>\n<body>\n<h1>Ikaros Web View</h1>\n<p>The following views are available:</p>\n<ul>");
        for (XMLElement * xml_view = xml->GetElement("view"); xml_view != NULL; xml_view = xml_view->next->FindElement("view"), v++)
        {
            char * n = xml_view->GetAttribute("name");
            if (n == NULL)
                socket->Send("   <li><a href=\"view%d.svg\">View %d</a></li>\n", v, v);
            else
                socket->Send("   <li><a href=\"%s.svg\">%s</a></li>\n", n, n);
        }

        socket->Send("</ul>\n</body>\n</html>");
    }
}



void
WebUI::ReadXML(XMLDocument * xmlDoc)
{
    if (xmlDoc == NULL)
    {
        return;
    }

    if (xmlDoc->xml == NULL)
    {
        k->Notify(msg_fatal_error, "WebUI: xmlDoc missing\n");
        return;
    }

    xml = xmlDoc->xml->FindElement("group");

    if (xml == NULL)
    {
        xml = xmlDoc->xml->FindElement("network");
        if (xml != NULL)
            xml = xml->GetElement("views");
    }

    // Catch exceptions here!!!

    if (xml == NULL)
    {
        k->Notify(msg_warning, "WebUI: No views found\n");	// add exception based error handling later
        return;
    }
}

#endif
