//
//  WebUI.h		Web UI code for the IKAROS project
//
//    Copyright (C) 2005-2007  Christian Balkenius
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//    See http://www.ikaros-project.org/ for more information.
//
//	Created August 31, 2005
//

#ifndef WEBUI
#define WEBUI

#include <stdio.h>
#include <stdarg.h>

#include "IKAROS.h"

#ifdef USE_SOCKET

#ifndef     WEBUIPATH
#define		WEBUIPATH	"Source/WebUI/UI/"
#endif

#define		PORT 8000

class WebUI;


class DataSource
{
public:
    DataSource *	next;
    char *			name;
    Module_IO *		module_io;

    DataSource(char * s, Module_IO * io, DataSource * n);
    ~DataSource();
};



class ModuleData
{
public:
    ModuleData *	next;
    char *			name;
    Module *		module;
    DataSource *	source;

    ModuleData(char * module_name, Module * m, ModuleData * n);
    ~ModuleData();

    void			AddSource(char * s, Module_IO * io);
};


const int ui_state_stop = 0;
const int ui_state_pause = 1;
const int ui_state_step = 2;
const int ui_state_run = 3;
const int ui_state_fast = 4;
const int ui_state_free = 5;


class WebUI
{
public:
    Kernel *		k;
    char *          webui_dir;
    int             port;
    ServerSocket *	socket;
    XMLElement *	xml;
    bool			debug_mode;
    bool			isRunning;
    long			tick;
    int             ui_state;

    ModuleData *	view_data;

    WebUI(Kernel * kernel);
    ~WebUI();

    void			AddDataSource(char * module, char * source);
    void			SendAllJSONData();
    void            SendMinimalJSONData();
    void			SendMenu(float x, float y, char * name);
    void			Send404();
    void			SendView(char * view);
    void			ReadXML(XMLDocument * xmlDoc);
    void			HandleHTTPRequest();
    void			Run();
};

#endif
#endif

