### Makefile fragment for the kernel
#
# 2007, Jan Moren
#

### a stack to keep track of the current directory

sp 		:= $(sp).x
dirstack_$(sp)	:= $(d)
d		:= $(dir)

### Object files, source files, includes and libraries
#
# edit as needed - don't forget to prefix with "$(d)" as 
# we are building from the top directory
#

# Object files to be built and linked
IKAROSOBJ	:= $(d)/IKAROS.o 
MAINOBJ		:= $(d)/main.o 
MATHOBJ		:= $(d)/IKAROS_Math.o 
SERIALOBJ	:= $(d)/IKAROS_Serial.o 
SIGNALOBJ	:= $(d)/IKAROS_Signal.o 
SOCKETOBJ	:= $(d)/IKAROS_Socket.o 
THREADSOBJ	:= $(d)/IKAROS_Threads.o 
TIMEROBJ	:= $(d)/IKAROS_Timer.o 
UTILSOBJ	:= $(d)/IKAROS_Utils.o 
XMLOBJ		:= $(d)/IKAROS_XML.o 

# Source files we depend on - don't forget the prefix
IKAROSSRC	:= $(d)/IKAROS.cc \
		IKAROS.h
MAINSRC		:= $(d)/main.cc \
		Modules/Modules.h UserModules/UserModules.h \
		WebUI/WebUI.h IKAROS.h
MATHSRC		:= $(d)/IKAROS_Math.cc   $(d)/IKAROS_Math.h \
		$(d)/main.o
SERIALSRC	:= $(d)/IKAROS_Serial.cc $(d)/IKAROS_Serial.h \
		$(d)/main.o
SIGNALSRC	:= $(d)/IKAROS_Signal.cc \
		$(d)/main.o
SOCKETSRC	:= $(d)/IKAROS_Socket_BSD.cc $(d)/IKAROS_Socket.h  \
		$(d)/main.o

# completely ugly, non-portable way of semi-fixing the minimal build 
ifneq (,$(findstring DMINIMAL, $(DEFS))) 
THREADSSRC	:= $(d)/IKAROS_Threads_none.cc $(d)/IKAROS_Threads.h  \
		$(d)/main.o
TIMERSRC	:= $(d)/IKAROS_Timer_none.cc $(d)/IKAROS_Timer.h  \
		$(d)/main.o
else
THREADSSRC	:= $(d)/IKAROS_Threads_pthread.cc $(d)/IKAROS_Threads.h  \
		$(d)/main.o
TIMERSRC	:= $(d)/IKAROS_Timer_POSIX.cc $(d)/IKAROS_Timer.h  \
		$(d)/main.o
endif
UTILSSRC	:= $(d)/IKAROS_Utils.cc $(d)/IKAROS_Utils.h  \
		$(d)/main.o
XMLSRC		:= $(d)/IKAROS_XML.cc $(d)/IKAROS_XML.h  \
		$(d)/main.o

# Additional libraries
# completely ugly, non-portable way of semi-fixing the minimal build 
ifneq (,$(findstring DMINIMAL, $(DEFS))) 
LIB		:=
else
LIB		:= -lgslcblas -lpthread 
endif

# Additional includes (we already add the current dir below)
INC		:= 


# propagate 
OBJ		:= $(IKAROSOBJ)  $(MAINOBJ) $(MATHOBJ)  $(SERIALOBJ)  \
		$(SOCKETOBJ)  $(THREADSOBJ)  $(TIMEROBJ)  $(UTILSOBJ) \
		$(XMLOBJ) 

OBJS		:= $(OBJS) $(OBJ)

LIBS		:= $(LIBS) $(LIB)
INCS		:= $(INCS) $(INC) -I$(d)
CLEAN		:= $(CLEAN) $(OBJ) 

# build rules - generic
$(IKAROSOBJ) 	: $(IKAROSSRC)

$(MAINOBJ) 	: $(MAINSRC)

$(MATHOBJ) 	: $(MATHSRC)

$(SERIALOBJ) 	: $(SERIALSRC)

$(SEIGNALOBJ) 	: $(SIGNALSRC)

# These don't quite fit the standard rule (the cc file and the 
# object file name don't match)
$(SOCKETOBJ) 	: $(SOCKETSRC)
		$(CC) $(CXXFLAGS) -o $@  $(INCS) -c $<

$(THREADSOBJ) 	: $(THREADSSRC)
		$(CC) $(CXXFLAGS) -o $@  $(INCS) -c $<

$(TIMEROBJ) 	: $(TIMERSRC)
		$(CC) $(CXXFLAGS) -o $@  $(INCS) -c $<

$(UTILSOBJ) 	: $(UTILSSRC)

$(XMLOBJ) 	: $(XMLSRC)


### Add additional build directories if you want/need
#
# Any place is fine as long as the path in "dir" is correct
# from the top Makefile

# dir             := $(d)/Userdir
# include         $(dir)/Rules.mk


### Pop the directory stack and move on

d		:= $(dirstack_$(sp))
sp		:= $(basename $(sp))

